
StageItem jQuery Plugin - easy Wix-like stage items
================================
This jQuery Plugin provides an easy way to make elements draggable, resizable and styled in a wix-like focusBox

![stage-item | Wix-like stage items](./common/example.png?raw=true)

## Getting Started

### Importing all necessary dependencies
This plugin requires **jQuery** and **jQuery-UI** to be included in your project, as it depends on them.

### Download the .js file
Download or copy the code from the file into your project.

#### Or

### Use a CDN
[Development version](https://cdn.jsdelivr.net/gh/wix-prototypers/prototypers_stage-item@1.0.0/stageItem.js)\
[Minified version](https://cdn.jsdelivr.net/gh/wix-prototypers/prototypers_stage-item@1.0.0/stageItem.min.js)

### Including it on your page
Include jQuery, jQuery-UI and the plugin on a page. Then select an element and call the `stageItem` method.

```html
<div class="item"></div>

<script src="jquery.js"></script>
<script src="jquery-ui.js"></script>
<script src="jquery-ui-stage-item.js"></script>

<script>
    $(".item").stageItem();
</script>
```

## Options
### All options with default values.
```js
{
   indicationsContainer = 'body';
   mainColor: '#459fed',
   nestingHierarchy: true,
   onResizeStart: () => true,
   onResize: () => true,
   onResizeStop: () => true,
   onDragStart: () => true,
   onDrag: () => true,
   onDragStop: () => true
}
```

### indicationsContainer
Choose where the focus boxes will be appended once they are created.\
The default is to append them to the ```body``` element.

### mainColor.
Color for the focus box and the resizers outlines

### nestingHierarchy
When set to true, focus boxes will recieve a prop which indecates their original item nesting level:
```data-nesting-level="1"```

This can be used to assist in creating a heirarchy baseed logic, like parent first selection model.

### resizable/draggable callbacks
These functions will be called on the different stages of resizing/dragging elements.
Use them to add additional logic you want to be executed on a specific stage.

These functions don't need to return a value, but returning ```false``` will stop the stage from executing, effectively canceling it.
Also, the ```event``` & ```ui``` objects from the corresponding resizable/draggable functions are passed automatically to the callbacks, and can be used inside them.

Resize functions:
```js
   onResizeStart,
   onResize, 
   onResizeStop ,
```

Drag functions:
```js
   onDragStart,
   onDrag, 
   onDragStop,
```


### Example:
```js
  $(".item").stageItem({
      mainColor: '#BADA55',
      onResizeStart: () => {console.warn('resizing is not allowed!!'); return false},
      onDragStop: (e,ui)=> console.log(`dragged element is: ${ui.helper}`)
  });
```
