($ => {
  $.fn.stageItem = function (options) {
    const defaults = new (function () {
      this.indicationsContainer = 'body';
      this.mainColor = '#459fed';
      this.nestingHierarchy = true;
      this.onResizeStart = () => true;
      this.onResize = () => true;
      this.onResizeStop = () => true;
      this.onDragStart = () => true;
      this.onDrag = () => true;
      this.onDragStop = () => true;
    })();

    let settings = $.extend({}, defaults, options || {});
    let styling = $('style#StageItem--styling');

    if (this.length > 1) {
      this.each(function () {
        $(this).stageItem(options);
      });
      return this;
    }

    const init = () => {
      $('html').on('click', e => {
        $('.focus-box').removeClass('stage-item-focused');
      });

      this.addClass('stage-item');
      if (this.parent().css('position').toLowerCase() === 'static') {
        this.parent().css('position', 'relative');
      }

      createFocusBox();

      if (!styling.length) {
        styling = createStyling();
      }

      return this;
    };

    // private members
    let isHorizontalResize;
    let cancel = false;

    // private methods
    const createStyling = () => {
      const style = document.createElement('style');
      style.setAttribute('id','StageItem--styling')
      style.innerHTML = `
        :root{
          --main-color: ${settings.mainColor};
          --offset-size: -5px;
        }

        .focus-box{
          opacity:0;
          border: 1px solid var(--main-color);
          cursor:move;
        }

        .focus-box:hover,.focus-box.stage-item-focused{
          opacity:1;
        }

        .focus-box > .ui-resizable-handle {
          display: none !important;
          box-sizing: border-box;
          width: 8px !important;
          height: 8px !important;
          background: white;
          border-radius:50%;
          border: 1px solid var(--main-color);
          position: absolute;
        }

        .focus-box:not(.stage-item-isDragged):not(.stage-item-isResized).stage-item-focused > .ui-resizable-handle{
          display: block !important;
          z-index: 10;
        }

        .focus-box .ui-resizable-n {
          left: 50%;
          top: var(--offset-size);
          cursor: ns-resize;
          transform: translateX(-50%);
        }

        .focus-box .ui-resizable-s {
          left: 50%;
          bottom: var(--offset-size);
          cursor: ns-resize;
          transform: translateX(-50%);
        }

        .focus-box .ui-resizable-w {
          left: var(--offset-size);
          top: 50%;
          cursor: ew-resize;
          transform: translateY(-50%);
        }

        .focus-box .ui-resizable-e {
          right: var(--offset-size);
          top: 50%;
          cursor: ew-resize;
          transform: translateY(-50%);
        }

        .focus-box .ui-resizable-nw {
          left: var(--offset-size);
          top: var(--offset-size);
          cursor: nwse-resize;
        }

        .focus-box .ui-resizable-ne {
          right: var(--offset-size);
          top: var(--offset-size);
          cursor: nesw-resize;
        }

        .focus-box .ui-resizable-sw {
          left: var(--offset-size);
          bottom: var(--offset-size);
          cursor: nesw-resize;
        }

        .focus-box .ui-resizable-se {
          right: var(--offset-size);
          bottom: var(--offset-size);
          cursor: nwse-resize;
        }
        `;

      return $('head').append($(style));
    };

    const createFocusBox = () => {
      const $item = $(this);
      const uid = '_' + Math.random().toString(36).substr(2, 9);

      const nestingLevel = settings.nestingHierarchy
        ? $item.parents('.stage-item').length + 1
        : null;

      $item.attr('id', uid);
      $item.css('position', 'absolute');

      const $focusBox = $(`<div data-orig-id="${uid}" class="draggable resizable"></div>`);

      $focusBox
        .addClass('focus-box')
        .attr('data-nesting-level', nestingLevel || 'false')
        .css({
          position: 'absolute',
          width: `${$item[0].offsetWidth}px`,
          height: `${$item[0].offsetHeight}px`,
          left: `${$item.offset().left}px`,
          top: `${$item.offset().top}px`,
        })
        .resizable({
          handles: ' n, e, s, w, ne, se, sw, nw',
          start: (e, ui) => {
            if (!settings.onResizeStart(e, ui)) return false;
            isHorizontalResize =
              $(e.toElement)[0].classList.toString().includes('resizable-e') ||
              $(e.toElement)[0].classList.toString().includes('resizable-w');

            $('.focus-box').removeClass('stage-item-focused');
            $(ui.helper).addClass('stage-item-isResized').addClass('stage-item-focused');
          },
          resize: (e, ui) => {
            if (settings.onResize(e, ui) === false) {
              return false;
            }
            trackPosChange($(ui.element), isHorizontalResize);
          },
          stop: (e, ui) => {
            if (settings.onResizeStop(e, ui) === false) {
              return false;
            }
            $(ui.helper).removeClass('stage-item-isResized');
          },
        })
        .draggable({
          start: (e, ui) => {
            if (settings.onDragStart(e, ui) === false) {
              return false;
            }
            $(ui.helper).addClass('stage-item-isDragged');
          },
          drag: (e, ui) => {
            if (settings.onDrag(e, ui) === false) {
              return false;
            }
            trackPosChange($(ui.helper));
          },
          stop: (e, ui) => {
            if (settings.onDragStop(e, ui) === false) {
              return false;
            }
            $(ui.helper).removeClass('stage-item-isDragged');
          },
        })
        .click(e => {
          e.preventDefault();
          e.stopPropagation();
          $('.focus-box').removeClass('stage-item-focused');

          $(e.currentTarget).addClass('stage-item-focused');
        })
        .disableSelection();

      $(settings.indicationsContainer).append($focusBox);
    };

    const trackPosChange = ($focusBox, isHorizontalResize = false) => {
      const $origEl = $(`#${$focusBox.attr('data-orig-id')}`);
      const $origElParent = $origEl.parent().is('body') ? $('html') : $origEl.parent();
      const $origElChildren = $origEl.children('.stage-item');

      $origEl.css({
        width: `${$focusBox.width()}px`,
        top: `${$focusBox.offset().top - $origElParent.offset().top}px`,
        left: `${$focusBox.offset().left - $origElParent.offset().left}px`,
      });

      if (isHorizontalResize) {
        $focusBox.css({
          height: `${$origEl[0].scrollHeight}px`,
        });
      } else {
        $origEl.css({
          height: `${$focusBox.height()}px`,
        });
      }

      if ($origElChildren.length) {
        updateChildrenFocusBox($origEl);
      }
    };

    const updateChildrenFocusBox = $origEl => {
      $origEl.children('.stage-item').each(function () {
        const $childItem = $(this);
        const $childChildren = $childItem.children('.stage-item');
        const $focusBox = $(`[data-orig-id=${$childItem.attr('id')}]`);

        $focusBox.css({
          width: `${$childItem.width()}px`,
          top: `${$childItem.offset().top}px`,
          left: `${$childItem.offset().left}px`,
        });

        if ($childChildren.length) {
          updateChildrenFocusBox($childItem);
        }
      });
    };

    return init();
  };
})(jQuery);
